from django.shortcuts import render
from xross.toolbox import xross_view, xross_listener

from .utils import fetch_data


def inspect(request):
    """Is triggered by AJAX to get actual data from GitHub.

    :param request:
    """
    results = fetch_data(request.POST['username'], request.POST['token'])
    return render(request, 'issueinspector/sub_issues.html', {'repos': results})


@xross_view(inspect)
def inspector_main(request):
    """Main entry point. Link it to URL whatever you like.

    :param request:
    """
    xross_listener(http_method='POST')
    return render(request, 'issueinspector/index.html')



from datetime import datetime
from concurrent.futures import ThreadPoolExecutor

import requests


def fetch_data(username, token):
    """Fetches data from GitHub.

    :param str username: GitHub username
    :param str token: Access token
    :return:
    """

    def get(url):
        response = requests.get(url, auth=(username, token))
        return response.json()

    status_warning = 'warning'
    status_danger = 'danger'

    def process_repo(repo):

        def contribute_to_relevant_issues():
            relevant_issues.append({
                'id': issue_id,
                'number': issue_number,
                'title': issue_title,
                'url': issue_url,
                'date': datetime.strptime(issue_date.rstrip('Z'), '%Y-%m-%dT%H:%M:%S'),
                'status': issue_status,
                'hint': issue_hint,
                'comment': issue_comment,
                'tags': issue_tags,
            })

        if not repo['open_issues_count']:
            return

        if repo['fork']:
            return

        relevant_issues = []

        url_issues = '%s?state=open&per_page=100' % repo['issues_url'].replace('{/number}', '')
        for issue in get(url_issues):
            issue_id = issue['number']
            issue_number = issue['number']
            issue_title = issue['title']
            issue_url = issue['html_url']
            issue_date = issue['updated_at']
            issue_status = status_danger
            issue_comment = ''
            issue_tags = [
                {'color': label['color'], 'name': label['name']}
                for label in issue['labels']]

            if not issue['comments']:
                issue_hint = 'Issue has not comments. Please comment on it.'
                contribute_to_relevant_issues()
                continue

            comments = get('%s?per_page=100' % issue['comments_url'])

            relevant_comment = comments[-1]
            commenter = relevant_comment['user']['login']

            if commenter == username:
                issue_hint = 'You\'re the last commenter. Please review the issue.'
                issue_status = status_warning
            else:
                issue_hint = 'You\'re not the last commenter. Dialogue awaits for your answer.'

            issue_date = relevant_comment['updated_at']
            issue_comment = relevant_comment['body']
            contribute_to_relevant_issues()

        result = {
            'id': repo['id'],
            'name': repo['name'],
            'issues': relevant_issues
        }

        return result

    url_repos = 'https://api.github.com/users/%s/repos' % username
    repos = get(url_repos)

    with ThreadPoolExecutor(max_workers=6) as executor:
        results = [repo_data for repo_data in executor.map(process_repo, repos) if repo_data]

    return results



import os
import io
from setuptools import setup

from issueinspector import VERSION


PATH_BASE = os.path.dirname(__file__)
PATH_BIN = os.path.join(PATH_BASE, 'bin')

SCRIPTS = None
if os.path.exists(PATH_BIN):
    SCRIPTS = [os.path.join('bin', f) for f in os.listdir(PATH_BIN) if os.path.join(PATH_BIN, f)]


def get_readme():
    # This will return README (including those with Unicode symbols).
    with io.open(os.path.join(PATH_BASE, 'README.rst')) as f:
        return f.read()


setup(
    name='django-issueinspector',
    version='.'.join(map(str, VERSION)),
    url='https://github.com/idlesign/django-issueinspector',

    description='Issue inspector for your GitHub repositories',
    long_description=get_readme(),
    license='BSD 3-Clause License',

    author='Igor `idle sign` Starikov',
    author_email='idlesign@yandex.ru',

    packages=['issueinspector'],
    include_package_data=True,
    zip_safe=False,

    install_requires=['requests', 'django-xross'],
    scripts=SCRIPTS,

    classifiers=[
        # As in https://pypi.python.org/pypi?:action=list_classifiers
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'License :: OSI Approved :: BSD License'
    ],
)

